/**
 * This class can take a variable number of parameters on the command
 * line. Program execution begins with the main() method. The class
 * constructor is not invoked unless an object of type 'CMain'
 * created in the main() method.
 */

import com.ms.com.*; 
import com.ms.win32.*; 
import com.ms.dll.*; 
import wbemdisp.*; 

public class CMain
{
	/**
	 * The main entry point for the application. 
	 *
	 * @param args Array of parameters passed to the application
	 * via the command line.
	 */
	
	// j/direct setting enalbing the use of a necessary win32 api call
	/** @dll.import("KERNEL32", auto) */
	private static native int GetEnvironmentVariable(String lpName,  // environment variable name
													 StringBuffer lpBuffer,  // buffer for variable value
													 int nSize);  // size of buffer	
	public static void main (String[] args)
	{
		try
		{	
			#if (DEBUG)
				// debugger;  // break for debugging
			#endif
							
			System.out.print("\nAcquiring EventLogs collection\n\n");
		
			ISWbemServices oSWbemSvcs = (ISWbemServices)Ole32.CoGetObject("winmgmts:{(Security)}", 
																		  null, ISWbemServices.iid);
			// note - Ole32.CoGetObject call == vb GetObject command pulling object from running object table (rot)
			#if (DEBUG)
				// debugger;  // break for debugging
				ISWbemSecurity oSWbemSecurity = oSWbemSvcs.getSecurity_();
				int iBuffer = oSWbemSecurity.getAuthenticationLevel();  // 6
				iBuffer = oSWbemSecurity.getImpersonationLevel();  // 3
				// oSWbemSecurity.setImpersonationLevel(wbemPrivilegeSecurity);
				// oSWbemSecurity.setImpersonationLevel(wbemPrivilegeBackup);
				iBuffer = oSWbemSecurity.getAuthenticationLevel();  // 6
				iBuffer = oSWbemSecurity.getImpersonationLevel();  // 3
			#endif

			ISWbemObjectSet oSWbemObjectSet = oSWbemSvcs.InstancesOf("Win32_NTEventlogFile", 0, null); 
	
			IEnumVariant oSWbemObjectSetEnum = (IEnumVariant)oSWbemObjectSet.get_NewEnum(); 
			int iNumToGet = oSWbemObjectSet.getCount(); 
			Variant[] avItems = new Variant[iNumToGet]; 
			int[] aiNumReturned = {0}; 
			oSWbemObjectSetEnum.Next(iNumToGet, avItems, aiNumReturned); 
    			
			#if (DEBUG)
				// debugger;  // break for debugging
			
				iBuffer = oSWbemObjectSet.getCount();  // on mydevc = 6
				
				/*
				// depricated calls
				String sSysteDrive = System.getenv("systemdrive");
				String sSystemRoot = System.getenv("systemroot");
				*/
				
				// jdirect work around since System.getenv is now depricated
				String sSystemDrive = "";
				int iSize = GetEnvironmentVariable("SYSTEMDRIVE", null, 0);
				if (0 != iSize) {
					StringBuffer sBuffer = new StringBuffer(iSize);
					int ir = GetEnvironmentVariable("SYSTEMDRIVE", sBuffer, iSize);
					if (iSize < ir) // ensure buffer was in fact big enough to support copy
						throw (new Error("Error - GetEnvironmentVariable call failed!\n"));
					else
						sSystemDrive = sBuffer.toString();
				}
				String sSystemRoot = "";
				iSize = GetEnvironmentVariable("SYSTEMROOT", null, 0);
				if (0 != iSize) {
					StringBuffer sBuffer = new StringBuffer(iSize);
					int ir = GetEnvironmentVariable("SYSTEMROOT", sBuffer, iSize);
					if (iSize < ir) // ensure buffer was in fact big enough to support copy
						throw (new Error("Error - GetEnvironmentVariable call failed!\n"));
					else
						sSystemRoot = sBuffer.toString();
				}
				
				if ("" == sSystemDrive || "" == sSystemRoot) 
					throw (new Error("Error - failed to acquire systemdrive and/or systemroot environment values!\n"));
				
			    String sSystemFld = sSystemRoot.substring(sSystemDrive.length() + 1);
			    String sEvntLogFile = "AppEvent.evt";  // AppEvent.evt | DnsEvent.evt | File Rep.evt | NtDs.evt | NtFrs.evt | SecEvent.evt | SysEvent.evt
			    // Path_ properties of the form "\\MYDEVC\root\cimv2:Win32_NTEventlogFile.Name="C:\\OS\\system32\\config\\AppEvent.Evt""
			    // but the hostname and namespace path can be left out, note the \\ path separators and value quote wrappers are necessary
			    // format is "\\<Server>\<Namespace>:<Class>.Name="<Path_.Path>"
			    String sBufferDbg = "Win32_NTEventlogFile.Name=\"" + sSystemDrive + "\\\\" + sSystemFld + "\\\\system32\\\\config\\\\" + sEvntLogFile + "\"";
			    ISWbemObject oSWbemObjectDbg = oSWbemObjectSet.Item(sBufferDbg, 0);  // get specific item from SWbemObjectSet
				// oSWbemObject.ClearEventlog;  // cannot call IDispatch::Invoke method directly in vj++
				// therefore using alternate approach 2
				ISWbemMethodSet oSWbemMethodSetDbg = oSWbemObjectDbg.getMethods_();
				ISWbemMethod oSWbemMethodDbg = oSWbemMethodSetDbg.Item("ClearEventlog", 0);
				ISWbemObject oSWbemInParameters = oSWbemMethodDbg.getInParameters();
				ISWbemObject oSWbemOutParametersDbg = oSWbemObjectDbg.ExecMethod_("ClearEventlog", oSWbemInParameters, 0, null);
				// Variant vr = oSWbemOutParameters.???();
				// System.out.println(vr.toString());
				
				sEvntLogFile = "SysEvent.evt";  // AppEvent.evt | DnsEvent.evt | File Rep.evt | NtDs.evt | NtFrs.evt | SecEvent.evt | SysEvent.evt
				sBufferDbg = "Win32_NTEventlogFile.Name=\"" + sSystemDrive + "\\\\" + sSystemFld + "\\\\system32\\\\config\\\\" + sEvntLogFile + "\"";
				oSWbemObjectDbg = oSWbemObjectSet.Item(sBufferDbg, 0);  // get specific item from SWbemObjectSet
				// oSWbemObject.ClearEventlog;  // cannot call IDispatch::Invoke method directly in vj++
				// therefore using alternate approach 2
				Variant vrDbg = Dispatch.call(oSWbemObjectDbg, "ClearEventlog"); 
				System.out.println("\tdispatch.Call(..., \"ClearEventlog\") returned the result " + vrDbg.toString()); 
			#endif
			  
			for (int iIndex = 0; iIndex < aiNumReturned[0]; iIndex++)
			{ 
				// Object oSWbemObject = avItems[iIndex].getDispatch();
				ISWbemObject oSWbemObject = (ISWbemObject)avItems[iIndex].getDispatch(); 			

				#if (DEBUG)
					// debugger;  // break for debugging
				    ISWbemProperty oSWbemProperty = oSWbemObject.getProperties_().Item("Caption", 0);  // i.e. ISWbemProperty Object
				    String sBuffer = oSWbemProperty.getValue().getString();  // i.e. c:\os\system32\config\appevent.evt"
					sBuffer = oSWbemObject.getProperties_().Item("Caption", 0).getValue().getString();  // i.e. "c:\os\system32\config\appevent.evt"
				    sBuffer = oSWbemObject.getProperties_().Item("Name", 0).getValue().getString();  // i.e. "C:\OS\system32\config\AppEvent.Evt"
				    sBuffer = oSWbemObject.getProperties_().Item("FileName", 0).getValue().getString();  // i.e. "appevent"
				    sBuffer = oSWbemObject.getProperties_().Item("LogfileName", 0).getValue().getString();  // i.e. "Application"
				    sBuffer = oSWbemObject.getPath_().getPath();  // i.e. "\\MYDEVC\root\cimv2:Win32_NTEventlogFile.Name="C:\\OS\\system32\\config\\AppEvent.Evt""
				    sBuffer = oSWbemObject.getPath_().getServer();  //  i.e. "MYDEVC"
				    sBuffer = oSWbemObject.getPath_().getNamespace();  // i.e. "root\cimv2"
				    sBuffer = oSWbemObject.getPath_().getDisplayName();  // i.e. "Win32_NTEventlogFile
				#endif
				 
				System.out.print("\tclearing " + oSWbemObject.getProperties_().Item("LogfileName", 0).getValue().getString() + " EventLog\n");
				Variant vr = Dispatch.call(oSWbemObject, "ClearEventlog"); 
				// System.out.println(vr.toString()); 
			} 
			
			throw (new Error("The operation completed successfully.\n"));  // depart without any errors
		}
		
		catch (Error oError)
		{
			#if (DEBUG)
				// oError.printStackTrace();
			#endif
			
			if (!oError.getMessage().endsWith("completed successfully.\n"))
				System.out.print(oError.getMessage());
			
			System.out.print("\nReleasing EventLogs collection\n");
		}
	}
}
